@@ -407,8 +407,12 @@ link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	unsigned int metric;
 	struct rtattr *rta;
 	struct rtmsg *rtm;
+#ifdef INET
 	struct rt rt;
-	char ifn[IF_NAMESIZE + 1];
+#endif
+#ifdef INET6
+	struct rt6 rt6;
+#endif
 
 	if (nlm->nlmsg_type != RTM_DELROUTE)
 		return 0;
@@ -421,7 +425,7 @@ link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	rtm = NLMSG_DATA(nlm);
 	if (rtm->rtm_type != RTN_UNICAST ||
 	    rtm->rtm_table != RT_TABLE_MAIN ||
-	    rtm->rtm_family != AF_INET)
+	    (rtm->rtm_family != AF_INET && rtm->rtm_family != AF_INET6))
 		return 1;
 	/* Ignore messages generated by us.
 	 * For some reason we get messages generated by us
@@ -438,32 +442,65 @@ link_route(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	rt.gate.s_addr = INADDR_ANY;
 	metric = 0;
 	while (RTA_OK(rta, len)) {
+		switch (rtm->rtm_family) {
+		case AF_INET:
+			switch (rta->rta_type) {
+			case RTA_DST:
+				memcpy(&rt.dest.s_addr, RTA_DATA(rta),
+				    sizeof(rt.dest.s_addr));
+				break;
+			case RTA_GATEWAY:
+				memcpy(&rt.gate.s_addr, RTA_DATA(rta),
+				    sizeof(rt.gate.s_addr));
+				break;
+			case RTA_OIF:
+				rt.iface = if_findindex(ctx,
+				    *(unsigned int *)RTA_DATA(rta));
+				break;
+			}
+			break;
+		case AF_INET6:
+			switch (rta->rta_type) {
+			case RTA_DST:
+				memcpy(&rt6.dest.s6_addr, RTA_DATA(rta),
+				    sizeof(rt6.dest.s6_addr));
+				break;
+			case RTA_GATEWAY:
+				memcpy(&rt6.gate.s6_addr, RTA_DATA(rta),
+				    sizeof(rt6.gate.s6_addr));
+				break;
+			case RTA_OIF:
+				rt6.iface = if_findindex(ctx,
+				    *(unsigned int *)RTA_DATA(rta));
+				break;
+			}
+			break;
+		}
 		switch (rta->rta_type) {
-		case RTA_DST:
-			memcpy(&rt.dest.s_addr, RTA_DATA(rta),
-			    sizeof(rt.dest.s_addr));
-			break;
-		case RTA_GATEWAY:
-			memcpy(&rt.gate.s_addr, RTA_DATA(rta),
-			    sizeof(rt.gate.s_addr));
-			break;
-		case RTA_OIF:
-			rt.iface = if_findindex(ctx,
-			    *(unsigned int *)RTA_DATA(rta));
-			break;
 		case RTA_PRIORITY:
 			metric = *(unsigned int *)RTA_DATA(rta);
 			break;
 		}
 		rta = RTA_NEXT(rta, len);
 	}
-	if (rt.iface != NULL) {
-		if (metric == rt.iface->metric) {
+
+	switch (rtm->rtm_family) {
 #ifdef INET
-			inet_cidrtoaddr(rtm->rtm_dst_len, &rt.net);
-			ipv4_routedeleted(ctx, &rt);
+		case AF_INET:
+			if (rt.iface != NULL && metric == rt.iface->metric) {
+				inet_cidrtoaddr(rtm->rtm_dst_len, &rt.net);
+				ipv4_routedeleted(ctx, &rt);
+			}
+			break;
+#endif
+#ifdef INET6
+		case AF_INET6:
+			if (rt6.iface != NULL && metric == rt6.iface->metric) {
+				ipv6_mask(&rt6.net, rtm->rtm_dst_len);
+				ipv6_routedeleted(ctx, &rt6);
+			}
+			break;
 #endif
-		}
 	}
 	return 1;
 }
@@ -861,9 +898,8 @@ static int
 nla_put_string(struct nlmsghdr *n, unsigned short maxlen,
     unsigned short type, const char *data)
 {
-	unsigned short len;
 	struct nlattr *nla;
-	size_t sl;
+	size_t len, sl;
 
 	sl = strlen(data) + 1;
 	len = NLA_ALIGN(NLA_HDRLEN + sl);
@@ -874,9 +910,9 @@ nla_put_string(struct nlmsghdr *n, unsigned short maxlen,
 
 	nla = (struct nlattr *)NLMSG_TAIL(n);
 	nla->nla_type = type;
-	nla->nla_len = len;
+	nla->nla_len = (unsigned short)len;
 	memcpy(NLA_DATA(nla), data, sl);
-	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + len;
+	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + (unsigned short)len;
 	return 0;
 }
 
@@ -888,7 +924,7 @@ gnl_parse(struct nlmsghdr *nlm, struct nlattr *tb[], int maxtype)
 	size_t len, rem;
 	int type;
 
-	memset(tb, 0, sizeof(*tb) * (maxtype + 1));
+	memset(tb, 0, sizeof(*tb) * ((unsigned int)maxtype + 1));
 	ghdr = NLMSG_DATA(nlm);
 	head = (struct nlattr *)((char *) ghdr + GENL_HDRLEN);
 	len = nlm->nlmsg_len - GENL_HDRLEN - NLMSG_HDRLEN;
@@ -962,7 +998,7 @@ _if_getssid(__unused struct dhcpcd_ctx *ctx, struct interface *ifp,
 
 out:
 	ifp->ssid[ifp->ssid_len] = '\0';
-	return ifp->ssid_len;
+	return (int)ifp->ssid_len;
 }
 
 static int
@@ -977,7 +1013,7 @@ if_getssid_nl80211(struct interface *ifp)
 		return -1;
 	memset(&nlm, 0, sizeof(nlm));
 	nlm.hdr.nlmsg_len = NLMSG_LENGTH(sizeof(struct genlmsghdr));
-	nlm.hdr.nlmsg_type = family;
+	nlm.hdr.nlmsg_type = (unsigned short)family;
 	nlm.hdr.nlmsg_flags = NLM_F_REQUEST;
 	nlm.ghdr.cmd = NL80211_CMD_GET_INTERFACE;
 	nla_put_32(&nlm.hdr, sizeof(nlm), NL80211_ATTR_IFINDEX, ifp->index);
@@ -994,7 +1030,7 @@ if_getssid(struct interface *ifp)
 
 	r = if_getssid_wext(ifp->name, ifp->ssid);
 	if (r != -1)
-		ifp->ssid_len = r;
+		ifp->ssid_len = (unsigned int)r;
 #ifdef HAVE_NL80211_H
 	else if (r == -1)
 		r = if_getssid_nl80211(ifp);
@@ -1508,7 +1544,7 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp, int own)
 	if (ifp == NULL)
 		ifname = "all";
 	else if (own) {
-		if (if_disable_autolinklocal(ctx, ifp->index) == -1)
+		if (if_disable_autolinklocal(ctx, (int)ifp->index) == -1)
 			syslog(LOG_DEBUG, "%s: if_disable_autolinklocal: %m",
 			    ifp->name);
 	}
