@@ -470,7 +470,9 @@ dumplease:
 	}
 #endif
 #ifdef INET6
-	if (dhcp6 && d6_state && d6_state->new) {
+	if (dhcp6 && d6_state &&
+	    (d6_state->new || d6_state->state == DH6S_DELEGATED))
+	{
 		n = dhcp6_env(NULL, NULL, ifp,
 		    d6_state->new, d6_state->new_len);
 		if (n > 0) {
